==> comedi-0.6.2.tgz: comedi-0.6.2/demo/ni_eeprom_dump.c <==
/*
   This little utility dumps the EEPROM of a
   National Instruments AT-MIO E series card and
   prints the information contained therein in
   a human-readable form.

   Information here is taken from the AT-MIO E
   series Register-Level Programmer Manual.  It
   doesn't give much information about what the
   CALDACs actually do, but it shouldn't be
   difficult to figure out what they do.
 */

#include <stdio.h>
#include <comedilib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <getopt.h>
#include <ctype.h>

/* This structure is taken directly from module/ni_E.c */
struct ni_boardtype_struct {
	char *name;
	int caldactype;
	int eepromlayout;
};

static struct ni_boardtype_struct ni_boardtypes[] =
{
	{"at-mio-16e-1", 1, 1},
	{"at-mio-16e-2", 1, 1},
	{"at-mio-16e-10", 1, 2},
	{"at-mio-16de-10", 1, 2},
	{"at-mio-64e-3", 1, 1},
	{"at-mio-16xe-50", 2, 3}
};
#define N_BOARDTYPES 6

void print_calib1(unsigned char *calib);
void print_calib2(unsigned char *calib);
void print_calib3(unsigned char *calib);
int get_boardtype(int eeprom511);
void do_rawdump(char *eeprom);
void ni_calibrate(int dev,int layout,char *eeprom);


int main(int argc, char *argv[])
{
	int i;
	int file;
	char *fn = NULL;
	comedi_trig it;
	unsigned char eeprom[512];
	sampl_t data;
	unsigned int chan;
	int boardtype;
	int raw = 0;
	int c;
	int err;

	fn = "/dev/comedi0";
	while (1) {
		c = getopt(argc, argv, "rf");
		if (c == -1)
			break;
		switch (c) {
		case 'r':
			raw = 1;
			break;
		case 'f':
			fn = argv[optind];
			break;
		default:
			printf("bad option\n");
			exit(1);
		}
	}

	file = open(fn, O_RDWR);
	if (file < 0) {
		perror(fn);
		exit(0);
	}
	it.subdev = 6;
	it.mode = 0;
	it.flags = 0;
	it.n_chan = 1;
	it.chanlist = &chan;
	it.data = &data;
	it.n = 1;
	it.trigsrc = 0;
	it.trigvar = 0;
	it.trigvar1 = 0;

	for (i = 0; i < 512; i++) {
		chan = CR_PACK(i, 0, 0);
		if ((err = ioctl(file, COMEDI_TRIG, &it)) < 0) {
			perror("ioctl");
			printf("%d\n", errno);
			exit(1);
		}
		eeprom[i] = data;
	}

	boardtype = get_boardtype(eeprom[511]);
	ni_calibrate(file,ni_boardtypes[boardtype].eepromlayout,eeprom);

	it.subdev = 0;
	it.mode = 0;
	it.flags = 0;
	it.n_chan = 1;
	it.chanlist = &chan;
	it.data = &data;
	it.n = 1;
	it.trigsrc = 0;
	it.trigvar = 0;
	it.trigvar1 = 0;

	for (i = 0; i < 8; i++) {
		chan=CR_PACK(i,0,3);
		if ((err = ioctl(file, COMEDI_TRIG, &it)) < 0) {
			perror("ioctl");
			printf("%d\n", errno);
			exit(1);
		}
		printf("%d\n",data);
	}



	exit(0);

	if (raw) {
		do_rawdump(eeprom);
		exit(0);
	}
	printf("Board code:     0x%02x\n", eeprom[511]);
	printf("Revision:       0x%02x\n", eeprom[510]);
	printf("Sub-revision:   0x%02x\n", eeprom[509]);
	printf("\nFactory Calibration:  (%d/%d/%d)\n", eeprom[507], eeprom[506], eeprom[508]);

	boardtype = get_boardtype(eeprom[511]);
	switch (ni_boardtypes[boardtype].eepromlayout) {
	case 1:
		print_calib1(eeprom + 409);
		printf("\nUser Calibration 1:\n");
		print_calib1(eeprom + 371 + 1 - 18);
		printf("\nUser Calibration 2:\n");
		print_calib1(eeprom + 371 + 1 - 18 * 2);
		printf("\nUser Calibration 3:\n");
		print_calib1(eeprom + 371 + 1 - 18 * 3);
		printf("\nUser Calibration 4:\n");
		print_calib1(eeprom + 371 + 1 - 18 * 3);
		printf("\nUser Calibration 5:\n");
		print_calib1(eeprom + 371 + 1 - 18 * 3);
		break;
	case 2:
		print_calib2(eeprom + 406);
		printf("\nUser Calibration 1:\n");
		print_calib2(eeprom + 371 + 1 - 19);
		printf("\nUser Calibration 2:\n");
		print_calib2(eeprom + 371 + 1 - 19 * 2);
		printf("\nUser Calibration 3:\n");
		print_calib2(eeprom + 371 + 1 - 19 * 3);
		printf("\nUser Calibration 4:\n");
		print_calib2(eeprom + 371 + 1 - 19 * 3);
		printf("\nUser Calibration 5:\n");
		print_calib2(eeprom + 371 + 1 - 19 * 3);
		break;
	case 3:
		print_calib2(eeprom + 423);
		printf("\nUser Calibration 1:\n");
		print_calib2(eeprom + 371 + 1 - 16);
		printf("\nUser Calibration 2:\n");
		print_calib2(eeprom + 371 + 1 - 16 * 2);
		printf("\nUser Calibration 3:\n");
		print_calib2(eeprom + 371 + 1 - 16 * 3);
		printf("\nUser Calibration 4:\n");
		print_calib2(eeprom + 371 + 1 - 16 * 3);
		printf("\nUser Calibration 5:\n");
		print_calib2(eeprom + 371 + 1 - 16 * 3);
		break;
	default:
		printf("unknown EEPROM layout\n");
		break;
	}

	return 0;
}

int get_boardtype(int eeprom511)
{
	int i;

	for (i = 0; i < N_BOARDTYPES; i++) {
		if (eeprom511 == ni_boardtypes[i].boardcode)
			return i;
	}
	printf("Unknown board code.\n");
	return 0;
}

void print_calib1(unsigned char *calib)
{
	/*
	   this is the EEPROM calibration layout for
	   at-mio-16e-1, 16e-2, and 64e-3

	   one can only wonder why there are gratuitous
	   changes between these layouts...
	 */
	printf("  reference: 0x%02x%02x\n", calib[17], calib[16]);
	printf("  analog output (unipolar):\n");
	printf("    CALDAC 5:   0x%02x\n", calib[5]);
	printf("    CALDAC 6:   0x%02x\n", calib[3]);
	printf("    CALDAC 7:   0x%02x\n", calib[4]);
	printf("    CALDAC 8:   0x%02x\n", calib[2]);
	printf("    CALDAC 9:   0x%02x\n", calib[0]);
	printf("    CALDAC 10:  0x%02x\n", calib[1]);
	printf("  analog output (bipolar):\n");
	printf("    CALDAC 5:   0x%02x\n", calib[11]);
	printf("    CALDAC 6:   0x%02x\n", calib[9]);
	printf("    CALDAC 7:   0x%02x\n", calib[10]);
	printf("    CALDAC 8:   0x%02x\n", calib[8]);
	printf("    CALDAC 9:   0x%02x\n", calib[6]);
	printf("    CALDAC 10:  0x%02x\n", calib[7]);
	printf("  analog input:\n");
	printf("    CALDAC 1:   0x%02x\n", calib[14]);
	printf("    CALDAC 2:   0x%02x\n", calib[12]);
	printf("    CALDAC 3:   0x%02x\n", calib[13]);
	printf("    CALDAC 4:   0x%02x\n", calib[15]);
}

void print_calib2(unsigned char *calib)
{
	/*
	   this is the EEPROM calibration layout for
	   at-mio-16e-10, 16de-10
	 */
	printf("  reference: 0x%02x%02x\n", calib[18], calib[17]);
	printf("  analog output (unipolar):\n");
	printf("    CALDAC 5:   0x%02x\n", calib[5]);
	printf("    CALDAC 6:   0x%02x\n", calib[3]);
	printf("    CALDAC 7:   0x%02x\n", calib[4]);
	printf("    CALDAC 8:   0x%02x\n", calib[2]);
	printf("    CALDAC 9:   0x%02x\n", calib[0]);
	printf("    CALDAC 10:  0x%02x\n", calib[1]);
	printf("  analog output (bipolar):\n");
	printf("    CALDAC 5:   0x%02x\n", calib[11]);
	printf("    CALDAC 6:   0x%02x\n", calib[9]);
	printf("    CALDAC 7:   0x%02x\n", calib[10]);
	printf("    CALDAC 8:   0x%02x\n", calib[8]);
	printf("    CALDAC 9:   0x%02x\n", calib[6]);
	printf("    CALDAC 10:  0x%02x\n", calib[7]);
	printf("  analog input:\n");
	printf("    CALDAC 1:   0x%02x\n", calib[14]);
	printf("    CALDAC 2:   0x%02x\n", calib[12]);
	printf("    CALDAC 3:   0x%02x\n", calib[13]);
	printf("    CALDAC 4:   0x%02x\n", calib[16]);
	printf("    CALDAC 11:  0x%02x\n", calib[15]);
}

void print_calib3(unsigned char *calib)
{
	/*
	   this is the EEPROM calibration layout for
	   at-mio-16xe-50
	 */
	printf("  reference: 0x%02x%02x\n", calib[15], calib[14]);
	printf("  analog output:\n");
	printf("    CALDAC 4:   0x%02x\n", calib[2]);
	printf("    CALDAC 5:   0x%02x\n", calib[0]);
	printf("    CALDAC 6:   0x%02x\n", calib[3]);
	printf("    CALDAC 7:   0x%02x\n", calib[1]);
	printf("  analog input (unipolar):\n");
	printf("    CALDAC 0:   0x%02x\n", calib[5]);
	printf("    CALDAC 1:   0x%02x\n", calib[4]);
	printf("    CALDAC 2:   0x%02x\n", calib[6]);
	printf("    CALDAC 8:   0x%02x%02x\n", calib[8], calib[7]);
	printf("  analog input (unipolar):\n");
	printf("    CALDAC 0:   0x%02x\n", calib[10]);
	printf("    CALDAC 1:   0x%02x\n", calib[9]);
	printf("    CALDAC 2:   0x%02x\n", calib[11]);
	printf("    CALDAC 8:   0x%02x%02x\n", calib[13], calib[12]);
}

void do_rawdump(char *eeprom)
{
	int i, j, c;

	printf("NI AT-MIO E series EEPROM dump\n");
	for (i = 0; i < 32; i++) {
		for (j = 0; j < 16; j++) {
			printf("%02x", eeprom[i * 16 + j] & 0xff);
		}
		printf("  ");
		for (j = 0; j < 16; j++) {
			c = eeprom[i * 16 + j] & 0xff;
			printf("%c", isprint(c) ? c : '.');
		}
		printf("\n");
	}
}


void ni_write_caldac(int dev,int addr,int val)
{
	comedi_trig it;
	unsigned int chan=CR_PACK(addr,0,0);
	sampl_t data=val;

	it.subdev = 5;
	it.mode = 0;
	it.flags = 0;
	it.n_chan = 1;
	it.chanlist = &chan;
	it.data = &data;
	it.n = 1;
	it.trigsrc = 0;
	it.trigvar = 0;
	it.trigvar1 = 0;

	if(ioctl(dev,COMEDI_TRIG,&it)){
		perror("ni_write_caldac");
		exit(0);
	}
}

void ni_calibrate(int dev,int layout,char *eeprom)
{
	switch (layout) {
	case 1:
		ni_write_caldac(dev, 4, eeprom[424]);
		ni_write_caldac(dev, 1, eeprom[423]);
		ni_write_caldac(dev, 3, eeprom[422]);
		ni_write_caldac(dev, 2, eeprom[421]);
		break;
	case 2:
		ni_write_caldac(dev, 4, eeprom[422]);
		ni_write_caldac(dev, 11, eeprom[421]);
		ni_write_caldac(dev, 1, eeprom[420]);
		ni_write_caldac(dev, 3, eeprom[419]);
		ni_write_caldac(dev, 2, eeprom[418]);
		break;
	default:
	}
}

==> comedi-0.6.3.tgz: comedi-0.6.3/demo/ni_eeprom_dump.c <==
/*
   This little utility dumps the EEPROM of a
   National Instruments AT-MIO E series card and
   prints the information contained therein in
   a human-readable form.

   Information here is taken from the AT-MIO E
   series Register-Level Programmer Manual.  It
   doesn't give much information about what the
   CALDACs actually do, but it shouldn't be
   difficult to figure out what they do.
 */

#include <stdio.h>
#include <comedilib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <getopt.h>
#include <ctype.h>

/* This structure is taken directly from module/ni_E.c */
struct ni_boardtype_struct {
	char *name;
	int caldactype;
	int eepromlayout;
};

static struct ni_boardtype_struct ni_boardtypes[] =
{
	{"at-mio-16e-1", 1, 1},
	{"at-mio-16e-2", 1, 1},
	{"at-mio-16e-10", 1, 2},
	{"at-mio-16de-10", 1, 2},
	{"at-mio-64e-3", 1, 1},
	{"at-mio-16xe-50", 2, 3}
};
#define N_BOARDTYPES 6

void print_calib1(unsigned char *calib);
void print_calib2(unsigned char *calib);
void print_calib3(unsigned char *calib);
int get_boardtype(int eeprom511);
void do_rawdump(char *eeprom);
void ni_calibrate(int dev,int layout,char *eeprom);


int main(int argc, char *argv[])
{
	int i;
	int file;
	char *fn = NULL;
	comedi_trig it;
	unsigned char eeprom[512];
	sampl_t data;
	unsigned int chan;
	int boardtype;
	int raw = 0;
	int c;
	int err;

	fn = "/dev/comedi0";
	while (1) {
		c = getopt(argc, argv, "rf");
		if (c == -1)
			break;
		switch (c) {
		case 'r':
			raw = 1;
			break;
		case 'f':
			fn = argv[optind];
			break;
		default:
			printf("bad option\n");
			exit(1);
		}
	}

	file = open(fn, O_RDWR);
	if (file < 0) {
		perror(fn);
		exit(0);
	}
	it.subdev = 6;
	it.mode = 0;
	it.flags = 0;
	it.n_chan = 1;
	it.chanlist = &chan;
	it.data = &data;
	it.n = 1;
	it.trigsrc = 0;
	it.trigvar = 0;
	it.trigvar1 = 0;

	for (i = 0; i < 512; i++) {
		chan = CR_PACK(i, 0, 0);
		if ((err = ioctl(file, COMEDI_TRIG, &it)) < 0) {
			perror("ioctl");
			printf("%d\n", errno);
			exit(1);
		}
		eeprom[i] = data;
	}

	boardtype = get_boardtype(eeprom[511]);
	ni_calibrate(file,ni_boardtypes[boardtype].eepromlayout,eeprom);

	if (raw) {
		do_rawdump(eeprom);
		exit(0);
	}
	printf("Board code:     0x%02x\n", eeprom[511]);
	printf("Revision:       0x%02x\n", eeprom[510]);
	printf("Sub-revision:   0x%02x\n", eeprom[509]);
	printf("\nFactory Calibration:  (%d/%d/%d)\n", eeprom[507], eeprom[506], eeprom[508]);

	boardtype = get_boardtype(eeprom[511]);
	switch (ni_boardtypes[boardtype].eepromlayout) {
	case 1:
		print_calib1(eeprom + 409);
		printf("\nUser Calibration 1:\n");
		print_calib1(eeprom + 371 + 1 - 18);
		printf("\nUser Calibration 2:\n");
		print_calib1(eeprom + 371 + 1 - 18 * 2);
		printf("\nUser Calibration 3:\n");
		print_calib1(eeprom + 371 + 1 - 18 * 3);
		printf("\nUser Calibration 4:\n");
		print_calib1(eeprom + 371 + 1 - 18 * 3);
		printf("\nUser Calibration 5:\n");
		print_calib1(eeprom + 371 + 1 - 18 * 3);
		break;
	case 2:
		print_calib2(eeprom + 406);
		printf("\nUser Calibration 1:\n");
		print_calib2(eeprom + 371 + 1 - 19);
		printf("\nUser Calibration 2:\n");
		print_calib2(eeprom + 371 + 1 - 19 * 2);
		printf("\nUser Calibration 3:\n");
		print_calib2(eeprom + 371 + 1 - 19 * 3);
		printf("\nUser Calibration 4:\n");
		print_calib2(eeprom + 371 + 1 - 19 * 3);
		printf("\nUser Calibration 5:\n");
		print_calib2(eeprom + 371 + 1 - 19 * 3);
		break;
	case 3:
		print_calib2(eeprom + 423);
		printf("\nUser Calibration 1:\n");
		print_calib2(eeprom + 371 + 1 - 16);
		printf("\nUser Calibration 2:\n");
		print_calib2(eeprom + 371 + 1 - 16 * 2);
		printf("\nUser Calibration 3:\n");
		print_calib2(eeprom + 371 + 1 - 16 * 3);
		printf("\nUser Calibration 4:\n");
		print_calib2(eeprom + 371 + 1 - 16 * 3);
		printf("\nUser Calibration 5:\n");
		print_calib2(eeprom + 371 + 1 - 16 * 3);
		break;
	default:
		printf("unknown EEPROM layout\n");
		break;
	}

	return 0;
}

int get_boardtype(int eeprom511)
{
	int i;

return 2;
#if 0
	for (i = 0; i < N_BOARDTYPES; i++) {
		if (eeprom511 == ni_boardtypes[i].boardcode)
			return i;
	}
	printf("Unknown board code.\n");
	return 0;
#endif
}

double get_ref_voltage(int hi,int lo)
{
	short r;

	r=(hi&0xff)<<8 | (lo&0xff);

	return 5.0+0.001*r;
}

void print_calib1(unsigned char *calib)
{
	/*
	   this is the EEPROM calibration layout for
	   at-mio-16e-1, 16e-2, and 64e-3

	   one can only wonder why there are gratuitous
	   changes between these layouts...
	 */

	printf("  reference: %g\n", get_ref_voltage(calib[17],calib[16]));
	printf("  analog output (unipolar):\n");
	printf("    CALDAC 5:   0x%02x\n", calib[5]);
	printf("    CALDAC 6:   0x%02x\n", calib[3]);
	printf("    CALDAC 7:   0x%02x\n", calib[4]);
	printf("    CALDAC 8:   0x%02x\n", calib[2]);
	printf("    CALDAC 9:   0x%02x\n", calib[0]);
	printf("    CALDAC 10:  0x%02x\n", calib[1]);
	printf("  analog output (bipolar):\n");
	printf("    CALDAC 5:   0x%02x\n", calib[11]);
	printf("    CALDAC 6:   0x%02x\n", calib[9]);
	printf("    CALDAC 7:   0x%02x\n", calib[10]);
	printf("    CALDAC 8:   0x%02x\n", calib[8]);
	printf("    CALDAC 9:   0x%02x\n", calib[6]);
	printf("    CALDAC 10:  0x%02x\n", calib[7]);
	printf("  analog input:\n");
	printf("    CALDAC 1:   0x%02x\n", calib[14]);
	printf("    CALDAC 2:   0x%02x\n", calib[12]);
	printf("    CALDAC 3:   0x%02x\n", calib[13]);
	printf("    CALDAC 4:   0x%02x\n", calib[15]);
}

void print_calib2(unsigned char *calib)
{
	/*
	   this is the EEPROM calibration layout for
	   at-mio-16e-10, 16de-10
	 */

	printf("  reference: %g\n", get_ref_voltage(calib[18],calib[17]));
	printf("  analog output (unipolar):\n");
	printf("    CALDAC 5:   0x%02x\n", calib[5]);
	printf("    CALDAC 6:   0x%02x\n", calib[3]);
	printf("    CALDAC 7:   0x%02x\n", calib[4]);
	printf("    CALDAC 8:   0x%02x\n", calib[2]);
	printf("    CALDAC 9:   0x%02x\n", calib[0]);
	printf("    CALDAC 10:  0x%02x\n", calib[1]);
	printf("  analog output (bipolar):\n");
	printf("    CALDAC 5:   0x%02x\n", calib[11]);
	printf("    CALDAC 6:   0x%02x\n", calib[9]);
	printf("    CALDAC 7:   0x%02x\n", calib[10]);
	printf("    CALDAC 8:   0x%02x\n", calib[8]);
	printf("    CALDAC 9:   0x%02x\n", calib[6]);
	printf("    CALDAC 10:  0x%02x\n", calib[7]);
	printf("  analog input:\n");
	printf("    CALDAC 1:   0x%02x\n", calib[14]);
	printf("    CALDAC 2:   0x%02x\n", calib[12]);
	printf("    CALDAC 3:   0x%02x\n", calib[13]);
	printf("    CALDAC 4:   0x%02x\n", calib[16]);
	printf("    CALDAC 11:  0x%02x\n", calib[15]);
}

void print_calib3(unsigned char *calib)
{
	/*
	   this is the EEPROM calibration layout for
	   at-mio-16xe-50
	 */
	double ref;

	ref=5.0+0.001*(calib[15]*0x100+calib[14]);
	printf("  reference: %g\n", ref);
	printf("  analog output:\n");
	printf("    CALDAC 4:   0x%02x\n", calib[2]);
	printf("    CALDAC 5:   0x%02x\n", calib[0]);
	printf("    CALDAC 6:   0x%02x\n", calib[3]);
	printf("    CALDAC 7:   0x%02x\n", calib[1]);
	printf("  analog input (unipolar):\n");
	printf("    CALDAC 0:   0x%02x\n", calib[5]);
	printf("    CALDAC 1:   0x%02x\n", calib[4]);
	printf("    CALDAC 2:   0x%02x\n", calib[6]);
	printf("    CALDAC 8:   0x%02x%02x\n", calib[8], calib[7]);
	printf("  analog input (unipolar):\n");
	printf("    CALDAC 0:   0x%02x\n", calib[10]);
	printf("    CALDAC 1:   0x%02x\n", calib[9]);
	printf("    CALDAC 2:   0x%02x\n", calib[11]);
	printf("    CALDAC 8:   0x%02x%02x\n", calib[13], calib[12]);
}

void do_rawdump(char *eeprom)
{
	int i, j, c;

	printf("NI AT-MIO E series EEPROM dump\n");
	for (i = 0; i < 32; i++) {
		for (j = 0; j < 16; j++) {
			printf("%02x", eeprom[i * 16 + j] & 0xff);
		}
		printf("  ");
		for (j = 0; j < 16; j++) {
			c = eeprom[i * 16 + j] & 0xff;
			printf("%c", isprint(c) ? c : '.');
		}
		printf("\n");
	}
}


void ni_write_caldac(int dev,int addr,int val)
{
	comedi_trig it;
	unsigned int chan=CR_PACK(addr,0,0);
	sampl_t data=val;

	it.subdev = 5;
	it.mode = 0;
	it.flags = 0;
	it.n_chan = 1;
	it.chanlist = &chan;
	it.data = &data;
	it.n = 1;
	it.trigsrc = 0;
	it.trigvar = 0;
	it.trigvar1 = 0;

	if(ioctl(dev,COMEDI_TRIG,&it)){
		perror("ni_write_caldac");
		exit(0);
	}
}

void ni_calibrate(int dev,int layout,char *eeprom)
{
	switch (layout) {
	case 1:
		ni_write_caldac(dev, 4, eeprom[424]);
		ni_write_caldac(dev, 1, eeprom[423]);
		ni_write_caldac(dev, 3, eeprom[422]);
		ni_write_caldac(dev, 2, eeprom[421]);
		break;
	case 2:
		ni_write_caldac(dev, 4, eeprom[422]);
		ni_write_caldac(dev, 11, eeprom[421]);
		ni_write_caldac(dev, 1, eeprom[420]);
		ni_write_caldac(dev, 3, eeprom[419]);
		ni_write_caldac(dev, 2, eeprom[418]);
		break;
	default:
	}
}
